import sys

from ddc.utils import exec_cmd

__version__ = "0.3.3"
__package_name__ = "ddc"


def print_about_version():
    print("Devision Developers Cli")  # noqa
    print("Version: " + __version__)  # noqa


def upgrade_pip_version_to_latest():
    _exitcode, out = exec_cmd(
        sys.executable + " -m pip install {} -U".format(__package_name__),
        print_exec_cmd=False,
    )
    print(out)  # noqa
    if not _exitcode:
        print("Update OK")  # noqa


def check_pip_version():
    _exitcode, latest_version = exec_cmd(
        sys.executable + " -m pip install {}==random".format(__package_name__),
        print_exec_cmd=False,
    )
    latest_version = latest_version[: latest_version.find(")")]
    latest_version = latest_version.replace(" ", "").split(",")[-1]

    if latest_version == __version__:
        return True
    else:
        return False


if __name__ == "__main__":
    print(__version__)  # noqa



import os

from ddc.task_build_docker_image import __build_docker_image
from ddc.utils import stream_exec_cmd, __build_path, get_lang_tests_dir, md5

"""
Обяателен stage building при этом названия шагов билда обязаны быть названы как build_{ИМЯ_ЯЗЫКА_ИЗ_DDC}
"""


def __discovery_build_target(cwd: str, lang: str):
    target = "build_" + str(lang)
    dockerfile_ = cwd + "/Dockerfile"
    if os.path.isfile(dockerfile_):
        with open(dockerfile_, "r") as f:
            for line in f.readlines():
                if "FROM" in line and target in line:
                    return target
    return None


def _run_test(
    cwd: str, lang: str, save_test_results: bool = False, docker_image_tag: str = None
):
    """
    Замес с exit_code и StopIteration довольно серьезный и не надо рефакторить это,
    если ты четко не понимаешь зачем это сделано
    """
    all_output = []
    exit_code = -1

    image_tag = None
    try:
        if lang == "py":
            if not os.path.exists(cwd + "/tests"):
                all_output.append("DDC: Directory /tests is not exists")
                exit_code = 1
                raise StopIteration()

            if not docker_image_tag:
                build_target = __discovery_build_target(cwd, lang)
                exit_code, image_tag = __build_docker_image(
                    all_output, cwd, build_target
                )
                if exit_code:
                    raise StopIteration()
            else:
                image_tag = docker_image_tag

            rwmeta_path = __build_path("/.rwmeta/")

            docker_args = [
                "-v {rwmeta_path}:/root/.rwmeta".format(rwmeta_path=rwmeta_path),
                "-v {cwd}:/usr/app".format(cwd=cwd),
            ]

            meta_dev_token = os.environ.get("X-META-Developer-Settings")
            if meta_dev_token:
                # На серверах файлы с токенами не лежат, все делается через env
                docker_args.append(
                    "-e 'X-META-Developer-Settings=" + meta_dev_token + "'"
                )

            pytest_args = "--reruns 2"
            if save_test_results:
                pytest_args += (
                    # " -o junit_family=xunit2 --junitxml=/usr/app/test-reports/tests.xml"
                )
            exit_code, mix_output = stream_exec_cmd(
                "docker run --rm {docker_args} {image_tag} pytest ./tests {pytest_args}".format(
                    image_tag=image_tag,
                    docker_args=" ".join(docker_args),
                    pytest_args=pytest_args,
                )
            )

            all_output.append(mix_output)
            if exit_code:
                raise StopIteration()

        elif lang == "php":
            if not os.path.exists(cwd + "/tests"):
                all_output.append("DDC: Directory /tests is not exists")
                exit_code = 1
                raise StopIteration()

            if not docker_image_tag:
                build_target = __discovery_build_target(cwd, lang)
                exit_code, image_tag = __build_docker_image(
                    all_output, cwd, build_target
                )
            else:
                image_tag = docker_image_tag

            phpunit_args = "--configuration tests/phpunit.xml"
            docker_args = []
            exit_code, mix_output = stream_exec_cmd(
                "docker run --rm {docker_args} {image_tag} ./vendor/bin/phpunit {phpunit_args} ./tests".format(
                    image_tag=image_tag,
                    docker_args=" ".join(docker_args),
                    phpunit_args=phpunit_args,
                )
            )

            all_output.append(mix_output)
            if exit_code:
                raise StopIteration()

        elif lang == "java":
            # у java особая папка
            if not os.path.exists(cwd + "/src/test"):
                all_output.append("DDC: Directory /src/test is not exists")
                exit_code = 1
                raise StopIteration()
            if not docker_image_tag:
                # на java тестим two step build для прогона тестов
                build_target = __discovery_build_target(cwd, lang)
                exit_code, image_tag = __build_docker_image(
                    all_output, cwd, build_target
                )
            else:
                image_tag = docker_image_tag

            phpunit_args = ""
            docker_args = []
            all_output.append("DDC: Start tests")
            exit_code, mix_output = stream_exec_cmd(
                "docker run --rm {docker_args} {image_tag} ./gradlew clean test".format(
                    image_tag=image_tag,
                    docker_args=" ".join(docker_args),
                    phpunit_args=phpunit_args,
                )
            )
            #  docker build --target build_java -t ddc_local/tmp:3fce479d19d3c481a4fb0f413bb42a89_build_java /Users/arturgspb/IdeaProjects/meta-jdk8
            #  docker build --target gradle_cache -t ddc_local/tmp:3fce479d19d3c481a4fb0f413bb42a89_gradle_cache /Users/arturgspb/IdeaProjects/meta-jdk8
            #  docker run --rm ddc_local/tmp:3fce479d19d3c481a4fb0f413bb42a89_build_java ./gradlew clean test

            all_output.append(mix_output)
            if exit_code:
                raise StopIteration()
        else:
            all_output.append("No test for lang: " + lang)
    except StopIteration:
        pass
    finally:
        pass
        # if not docker_image_tag and image_tag:
        #     stream_exec_cmd("docker image rm " + image_tag)
    return exit_code, "\n".join(all_output)


def start_test(
    cwd: str, lang: str, save_test_results: bool = False, docker_image_tag: str = None
):
    if lang == "auto":
        new_output = []
        exit_code = 0
        lang_tests_dir = get_lang_tests_dir(cwd)
        if not lang_tests_dir:
            return -1, "No test for any auto lang"

        for item in lang_tests_dir:
            new_output.append(
                "Test lang: " + lang + ". Dir: " + item["dir_for_run_tests"]
            )
            exit_code, lang_output = _run_test(
                item["dir_for_run_tests"],
                item["lang"],
                save_test_results,
                docker_image_tag,
            )
            new_output.append(lang_output)
            if exit_code == 0:
                new_output.append("OK")
            else:
                new_output.append("Failure...")
                break
        return exit_code, "\n".join(new_output)
    else:
        _run_test(cwd, lang, save_test_results)



from ddc.utils import stream_exec_cmd, md5


def start_build_docker_image(cwd: str):
    all_output = []
    exit_code, image_tag = __build_docker_image(all_output, cwd)
    return exit_code, "\n".join(all_output), image_tag


def __build_docker_image(all_output, cwd: str, target: str = None):
    exit_code = -1
    if target:
        build_target = "--target " + str(target)
        image_tag: str = "ddc_local/tmp:" + md5(cwd) + "_" + str(target)
    else:
        build_target = ""
        image_tag: str = "ddc_local/tmp:" + md5(cwd)

    try:
        exit_code, mix_output = stream_exec_cmd(
            ("docker build {build_target} -t " + image_tag + " " + cwd).format(
                build_target=build_target
            )
        )
        print("mix_output = %s" % str(mix_output))
        if exit_code:
            all_output.append("DDC: Build docker image")
            all_output.append(mix_output)
            raise StopIteration()

        all_output.append(mix_output)
    except StopIteration:
        pass
    return exit_code, image_tag



import os

from ddc.utils import stream_exec_cmd, __build_path, run_if_time_has_passed


def build_bus_event(appcontent_dir):
    """
    Сборка шинных скриптов для последующего деплоя в Запускатор и запись в БД для самой меты
    """
    image_tag = "apisgarpun/pronto-meta-utils"

    def _pull():
        stream_exec_cmd("docker pull {image_tag}".format(image_tag=image_tag))

    run_if_time_has_passed("build_bus_event", 60, _pull)

    rwmeta_path = __build_path("/.rwmeta/")

    docker_args = [
        "-v {appcontent_dir}:/usr/appcontent".format(appcontent_dir=appcontent_dir),
        "-v {rwmeta_path}:/root/.rwmeta".format(rwmeta_path=rwmeta_path),
    ]

    meta_dev_token = os.environ.get("X-META-Developer-Settings")
    if meta_dev_token:
        # На серверах файлы с токенами не лежат, все делается через env
        docker_args.append("--env X-META-Developer-Settings='" + meta_dev_token + "'")

    return stream_exec_cmd(
        "docker run --rm {docker_args} {image_tag} build_bus_events".format(
            image_tag=image_tag, docker_args=" ".join(docker_args),
        )
    )



import time
import logging
from watchdog.observers import Observer
from watchdog.events import RegexMatchingEventHandler
import os
import threading
from ddc.utils import (
    write_ddc_file,
    get_path_ddc_file,
    exec_cmd_with_stream_output,
)

META_DOCKER_COMPOSE_FILENAME = "meta-docker-compose_{port}.yaml"

ACTUAL_DOCKER_COMPOSE_FILE = """
version: '3'
services:
  redis:
    image: redis:alpine
  meta:
    image: {docker_image}
    depends_on:
      - redis
    ports:
      - "{port}:8080"
    volumes:
      - ~/.rwmeta:/root/.rwmeta:delegated
      - ./meta_conf:/root/meta_conf:delegated
      - ./:/root/workspace/production:delegated
    environment:
      JAVA_OPTS: -Duser.timezone=Europe/Moscow -Xms1000m -Xmx2500m -Dcom.sun.management.jmxremote -Dcom.sun.management.jmxremote.port=9010 -Dcom.sun.management.jmxremote.authenticate=false -Dcom.sun.management.jmxremote.ssl=false -Djava.rmi.server.hostname=localhost
      META_CONFIG: /root/meta_conf/meta-loc-dev.yaml
      RELEASE_VERSION2: dev
      RELEASE_VERSION: local_dev
      META_APP_CONTENT_CONFIG_DIR: /root/meta_conf
      META_APP_CONTENT_WORKSPACE_DIR: /root/workspace
"""  # noqa


class GitAddEventHandler(RegexMatchingEventHandler):
    IMAGES_REGEX = [r".*\.html"]

    def __init__(self):
        super().__init__(self.IMAGES_REGEX)

    def on_moved(self, event):
        super(GitAddEventHandler, self).on_moved(event)

        self.__git_add_file(event.src_path)

    def on_created(self, event):
        super(GitAddEventHandler, self).on_created(event)
        self.__git_add_file(event.src_path)

    def __git_add_file(self, file_path: str):
        if file_path.endswith(".html"):
            os.system("git add " + file_path)
            logging.info("git add = %s" % str(file_path))


def start_add_file_watchdog(path):
    logging.info("Start new html files watchdog")

    event_handler = GitAddEventHandler()
    observer = Observer()
    observer.schedule(event_handler, path, recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()


class AddFilesThread(threading.Thread):
    def __init__(self, path):
        threading.Thread.__init__(self)
        self.path = path

    def run(self):
        start_add_file_watchdog(self.path)


def start_meta(cwd, port, docker_image):
    AddFilesThread(cwd).start()

    compose_filename = META_DOCKER_COMPOSE_FILENAME.format(port=port)
    compose_full_file_path = get_path_ddc_file(compose_filename)
    target_compose_content = ACTUAL_DOCKER_COMPOSE_FILE.format(
        port=port, docker_image=docker_image
    )
    write_ddc_file(compose_filename, target_compose_content)

    compose_base_cmd_base = [
        "docker-compose",
        "-f",
        "{compose_full_file_path}".format(compose_full_file_path=compose_full_file_path),
        "-p"
        "ddc_meta_{port}".format(port=port),
        "--project-directory={cwd}".format(cwd=cwd)
    ]

    exec_cmd_with_stream_output(compose_base_cmd_base + ["down"])
    try:
        exec_cmd_with_stream_output(compose_base_cmd_base + ["pull"])
        exec_cmd_with_stream_output(compose_base_cmd_base + ["up"])
    finally:
        exec_cmd_with_stream_output(compose_base_cmd_base + ["down"])



#!/usr/bin/env python
import io
import os
import sys

import setuptools
from setuptools import setup
from ddc import info

if sys.version_info < (3, 6, 0):
    print("Python 3.6+ is required")  # noqa
    exit(1)

here = os.path.abspath(os.path.dirname(__file__))
cur_dir = os.path.realpath(os.path.dirname(__file__))

with io.open(os.path.join(cur_dir, "requirements.txt")) as requirements_file:
    requirements = requirements_file.read().strip().split("\n")

setup(
    name=info.__package_name__,
    version=info.__version__,
    entry_points={"console_scripts": ["ddc=ddc.cli:main"]},
    description="Devision Developers Cli",
    long_description="",
    url="https://github.com/devision-io/ddc",
    author="Devision",
    author_email="info@devision.io",
    license="Apache 2.0",
    zip_safe=False,
    classifiers=[
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3 :: Only",
    ],
    install_requires=requirements,
    python_requires=">=3.6",
    packages=setuptools.find_packages(),
    package_data={"": ["LICENSE"]},
    platforms="Posix; MacOS X; Windows",
    include_package_data=True,
)
